
/*
C implementation of detect for ecg analysis.
In the main features it works as Detect.m

The "filter" used (filtering1984) is based on the work by C. Levkov et al Subtraction of 50 Hz interference from the electrocardiogram (1984)
The DOM method is implemented for the QR  detection (however there is a local search performed to find the exact location of the Rpeak)
The T peak is found with another local search performed in a positive time window starting form the R point.
The T end point is found using the Trapezium area method (Carlos R vazquez-Seisdedos 2011)




*/


#ifdef COSMOSRUN

	#define ECGWD 0 

#endif
#ifdef BLENDINGRUN
	
	#define ECGWD 1

#endif


double Detect(int *chandata, int *flag_for_over)

{
	/*Variables   */
	int X_d_f[lines-1];	
	int i, j;
	int idx_debug_for;
	int interval_number;
	int interval;
	int Q1point_loc;
	int Q2point_loc;
	int Qpoint_loc; 
	int T_offset;
	int search_interval1;
	int search_interval2;
	int mean_qt_value;
	double RR_forecasted; /*Measured in seconds */
	int flag_for_memory;
	FILE *fp_mat;
	FILE *fp_mat1;
	double t_end_window;
	FILE *fp_debug;
	FILE *fp_gp;
	static int gp_index=1;
	char str_gp_out1[60];
	char str_gp_out2[10];
	char EpsFormat[]=".eps";
	char *StrPlt;
	int Min4Arrow, Max4Arrow;
	
	if(FLAG_FOR_DEBUG)
	{	
		fp_debug=fopen("debugging.txt","a+");
		fprintf(fp_debug,"Inside the Detection... \n");
		fclose(fp_debug);
	}

	if(ECGWD)
	{
		strcpy(str_gp_out1,"DetectionPlots/EcgDetection");
	}
	else
	{
		strcpy(str_gp_out1,"BlendingCode/DetectionPlots/EcgDetection");
	}



	if(FLAG_FOR_MATLAB_PLOT || FLAG_FOR_GP_PLOT)
	{
		if(ECGWD)
		{
			fp_mat1=fopen("Results/unfiltered_data_to_plot.txt", "w+"); 				if(fp_mat1==NULL) THROW_ERROR
		}
		else
		{
			fp_mat1=fopen("BlendingCode/Results/unfiltered_data_to_plot.txt", "w+"); 				if(fp_mat1==NULL) THROW_ERROR
		}		
		for(i=0;i<lines;i++)
		{
			fprintf(fp_mat1,"%d\n", chandata[i]);
		}
		fclose(fp_mat1);
	}



	/* FILTERING  */
	




	if(FILTER_84)
	{    
	    filter_base_line(chandata);
	    filtering1984(chandata);						
        }
	if(FLAG_FOR_DEBUG)
	{	
		fp_debug=fopen("debugging.txt","a+");
		fprintf(fp_debug,"Filtering performed \n");
		fclose(fp_debug);
	}
	/* Saving filtered signal to file */
	if(FLAG_FOR_MATLAB_PLOT || FLAG_FOR_GP_PLOT)
	{
		if(ECGWD)
		{
			fp_mat1=fopen("Results/filtered_data.txt", "w+"); 				if(fp_mat1==NULL) THROW_ERROR
		}
		else
		{
			fp_mat1=fopen("BlendingCode/Results/filtered_data.txt", "w+"); 				if(fp_mat1==NULL) THROW_ERROR
		}
		for(i=0;i<lines;i++)
		{
			fprintf(fp_mat1,"%d\n", chandata[i]);
			//printf("%d \n",chandata[i]);
		}
		fclose(fp_mat1);
		Max4Arrow=peakfinder(chandata,lines);
		Min4Arrow=minpeakfinder(chandata,lines);
	}
	
	/*DIFFERENCE SIGNAL */

	if(FLAG_FOR_DEBUG)
	{	
		fp_debug=fopen("debugging.txt","a+");
		fprintf(fp_debug,"Computing difference signal \n");
		fclose(fp_debug);
	}	
	difference_first(chandata, X_d_f);				
	difference_second(X_d_f);

	
	
	if(FLAG_FOR_MATLAB_PLOT || FLAG_FOR_GP_PLOT)
	{
		if(ECGWD)
		{
			fp_mat1=fopen("Results/difference_data.txt", "w+"); 				if(fp_mat1==NULL) THROW_ERROR
		}
		else
		{
			fp_mat1=fopen("BlendingCode/Results/difference_data.txt", "w+"); 				if(fp_mat1==NULL) THROW_ERROR
		}		
		for(i=0;i<lines-1;i++)
		{
			fprintf(fp_mat1,"%d\n", X_d_f[i]);
		}
		fclose(fp_mat1);
	}

	
	/* WAVE DETECTION */

	/*finding R peaks*/
	
	interval=R_PEAK_INTERVAL;
	interval_number=0;
	for(i=0 ; i<(lines-interval) ; i=i+interval)			/* count in how many intervals the time line is devided */
	{
		interval_number++; 	
	}
	
	flag_for_memory=Rpeackdetect(X_d_f , interval, interval_number , chandata, flag_for_over);
				
	if(flag_for_memory==-2)	THROW_ERROR /*PROBLEM ALLOCATING MEMORY, 5 IS JUST A FLAG VALUE*/
	if(FLAG_FOR_DEBUG)
	{		
		fp_debug=fopen("debugging.txt","a+");
		fprintf(fp_debug,"Rpeak detected \n");
		fclose(fp_debug);
	}
	if(FLAG_FOR_MATLAB_PLOT)
	{
		if(ECGWD)
		{
			fp_mat=fopen("Results/plotting_lines.m" , "w+");				if(fp_mat==NULL)   THROW_ERROR  /*opening file to be plotted by matlab */
		}
		else
		{
			fp_mat=fopen("BlendingCode/Results/plotting_lines.m" , "w+");				if(fp_mat==NULL)   THROW_ERROR  /*opening file to be plotted by matlab */
		}
	}
	if(FLAG_FOR_GP_PLOT)
	{
		if(ECGWD)
		{
			fp_gp=fopen("DetectionPlots/EcgDetectionPlot.plt","w+");				if(fp_gp==NULL)		THROW_ERROR
		}
		else
		{
			fp_gp=fopen("BlendingCode/DetectionPlots/EcgDetectionPlot.plt","w+");				if(fp_gp==NULL)		THROW_ERROR
		}
	}

	if(R_points_loc_global[0]!=-1) /* execute the rest of the program only if R_peaks are found */
	{
			if(FLAG_FOR_DEBUG)
			{
				fp_debug=fopen("debugging.txt","a+");
				fprintf(fp_debug,"Inside the Q-T detction loop \n");
				fclose(fp_debug);
			}
		if(FLAG_FOR_MATLAB_PLOT)
		{	
			
			fprintf(fp_mat,"figure;\n subplot(2,1,1);\n hold on; xlabel('Sample Pt');\n ylabel('Voltage'); \n load('Results/filtered_data.txt'); \n plot(filtered_data);\n"); /*plotting the filtered signal*/
		}
		if(FLAG_FOR_GP_PLOT)
		{
			sprintf(str_gp_out2, "%d", gp_index++);
			StrPlt=strcat(strcat(str_gp_out1,str_gp_out2),EpsFormat);
			fprintf(fp_gp,"reset \n set term post eps font 'Helvetica,20' linewidth 2 \n set output '%s' \n set terminal postscript enhanced \n",StrPlt);
			fprintf(fp_gp,"set term postscript enhanced color \n set pointsize 1 \n set key top right \n set key font ',20'\n");
			fprintf(fp_gp, "set title 'Green: Q points. Red: T points'\n set key spacing .9 \n set style data linespoint \n set xlabel 'Sample Point'  \n  set ylabel 'Voltage'\n");
		}

		search_interval1=INTERVAL1;
		search_interval2=INTERVAL2;
	
		for(idx_debug_for=0; (idx_debug_for < QT_SAVED)&& ((R_points_loc_global[idx_debug_for])!=-1) ; idx_debug_for++)
		{	
			/*fp_debug=fopen("/home/scratch/patane/Cosmos1/prova/debugging.txt","a+");
			fprintf(fp_debug,"Eseguo i detects dei vari punti. idx_debug_for vale %d , QT_SAVED vale invece %d, e' idx_debug_for minore di QT_SAVED??? %d, ma QT-idx_debug_for vale %d , e' tale valore maggiore di zero?? %d, e l'altra disugualianza?? %d\n",idx_debug_for, QT_SAVED, idx_debug_for<QT_SAVED , QT_SAVED-idx_debug_for , (QT_SAVED-idx_debug_for)>0, (QT_SAVED-idx_debug_for)<=0 );
			fclose(fp_debug);	*/
			/*finding the Q point */
			Q1point_loc=Q_point_detect(chandata , R_points_loc_global[idx_debug_for]-2 , search_interval1);   /*computations of Q1s */
			Q2point_loc=Q_point_detect(chandata , R_points_loc_global[idx_debug_for]-2 , search_interval2);  /*computation of Q2s */
			Qpoint_loc=Q_deciding(Q1point_loc, Q2point_loc , chandata);		/*computation of Qs */
			/*printf("Qpoint_loc : %hd\n", Qpoint_loc); */
			/* T wave computations */
			t_end_window=(int)round( 0.33*((double)  RR_interval_discrete() )   );
			T_offset=Twave_detect(chandata, (R_points_loc_global[idx_debug_for]+T_START), (const int) t_end_window);	
			/*printf("T_offset : %hd\n", T_offset);		*/		
			/* QTI computation   */
			QT_intervals_global[global_index_QT]=T_offset-Qpoint_loc;
			global_index_QT++;			
			
			if(global_index_QT>=QT_SAVED)
				global_index_QT=0;
			
			if(FLAG_FOR_MATLAB_PLOT)
			{
				fprintf(fp_mat,"subplot(2,1,1);\n");
				fprintf(fp_mat, "line([%d %d], [-50 50], 'Color', [0 1 0]);\n", Qpoint_loc+1, Qpoint_loc+1);  /* plotting Q and T points */
				fprintf(fp_mat, "line([%d %d], [-50 50], 'Color', [1 0 0]);\n", T_offset+1, T_offset+1);
				fprintf(fp_mat,"subplot(2,1,2);\n");
				fprintf(fp_mat, "line([%d %d], [-50 50], 'Color', [0 1 0]);\n", Qpoint_loc+1, Qpoint_loc+1);  /* plotting Q and T points */
				fprintf(fp_mat, "line([%d %d], [-50 50], 'Color', [1 0 0]);\n", T_offset+1, T_offset+1);	
			}
			if(FLAG_FOR_GP_PLOT)
			{
				fprintf(fp_gp,"set arrow from %d,%d to %d,%d nohead lc rgb 'green'\n",Qpoint_loc+1,chandata[Min4Arrow],Qpoint_loc+1,chandata[Max4Arrow]);
				fprintf(fp_gp,"set arrow from %d,%d to %d,%d nohead lc rgb 'red'\n",T_offset+1,chandata[Min4Arrow], T_offset+1,chandata[Max4Arrow]);
			}
				
		
		}

	}
	if(FLAG_FOR_DEBUG)
	{	
		fp_debug=fopen("debugging.txt","a+");
		fprintf(fp_debug,"Done with ECG detection... \n");
		fclose(fp_debug);	
	}	

	if(FLAG_FOR_MATLAB_PLOT)
	{
		fprintf(fp_mat,"subplot(2,1,2);\n hold on\n ;xlabel('Heart beat num');\n ylabel('sec *10^{(-2)}');"); 
		fprintf(fp_mat,"subplot(2,1,2);\n hold on;\n load difference_data.txt ;\n plot(difference_data);\n");	 
		fprintf(fp_mat," hold off;");
		fprintf(fp_mat," disp 'Press Enter key to continue'; pause ; quit ");
		fclose(fp_mat);
		if(ECGWD)
		{
			system(" matlab -nodesktop -nosplash -r 'cd Results; plotting_lines' ");
		}
		else
		{
			system(" matlab -nodesktop -nosplash -r 'cd BlendingCode/Results; plotting_lines' ");
		}
		system(" clear ");		
	}
	if(FLAG_FOR_GP_PLOT)
	{
		if(FLAG_FOR_DEBUG)
		{	
			fp_debug=fopen("debugging.txt","a+");
			fprintf(fp_debug,"Almost there... \n");
			fclose(fp_debug);	
		}
		fprintf(fp_gp,"plot 'BlendingCode/Results/filtered_data.txt' ps 0 lc rgb 'blue' title 'Filtered Signal'");
		if(FLAG_FOR_DEBUG)
		{	
			fp_debug=fopen("debugging.txt","a+");
			fprintf(fp_debug,"W8... \n");
			fclose(fp_debug);	
		}
		fclose(fp_gp);
		if(FLAG_FOR_DEBUG)
		{	
			fp_debug=fopen("debugging.txt","a+");
			fprintf(fp_debug,"W8... \n");
			fclose(fp_debug);	
		}
		if(ECGWD)
		{
			system("gnuplot 'DetectionPlots/EcgDetectionPlot.plt'"); 
		}
		else
		{
			system("gnuplot 'BlendingCode/DetectionPlots/EcgDetectionPlot.plt'"); 			
		}
	}
			
		
		mean_qt_value=QT_wrap();
		RR_forecasted=pacing(mean_qt_value);
		fp_debug=fopen("debugging.txt","a+");
		if(FLAG_FOR_DEBUG)
		{	
			fprintf(fp_debug,"Returning the value to the caller function... \n");
			fclose(fp_debug);
		}
		return(RR_forecasted);	
}
