/*****************************************/
/*LIBRARY WRAP FOR THE BLENDING ALGORITHM*/
/*****************************************/



/* Standard libraries used in the code */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <limits.h>





/*INCLUDING LIBRARIES ACCORDING TO WHAT IS DEFINED IN THE CONFIGURATION LIBRARY */


#include "../SimulationConfig.h"



/*Frequency*/
#ifdef SAMPLE_FS_100		

	#include "SampleFreq/Frequency_100Hz.h"

#endif

#ifdef SAMPLE_FS_64

	#include "SampleFreq/Frequency_64Hz.h"	

#endif		

#ifdef SAMPLE_FS_128

	#include "SampleFreq/Frequency_128Hz.h"	

#endif		




/*Patient */
#ifdef ANDREA

	#include "Patient/Andrea.h"
	
#endif

#ifdef ALAN

	#include "Patient/Alan.h"

#endif

#ifdef R16272

	#include "Patient/R16272.h"

#endif




/*Signal Precision*/
#ifdef BIT_8_PRECISION

	#include "Signal/bit_signal_8.h"
 
#endif



#ifdef NSRDB_RECORD_PRECISION  
	
	#include "Signal/nsrdb_signal.h"

#endif



/*Accelerometers model */

#include "AccelerometerModel/AccelerometerModel.h"

/*Bending algorithm parameters */

#include "BlendingModel/BlendingModel.h"

#define BIN_MAX(a, b) ((a >= b) ? a : b) /* finding maximum between two numbers */

/* ERRORS USING POINTERS */
#define ERROR	{ printf("An error occured\n\n");exit(1);}

#define MEMORY	{ printf("Error occured while trying to allocating memory\n\n");exit(1);}
/*Random numbers generator   */
#define randomize srand((unsigned)time(NULL))

#define random(x) rand() % x



/* functions prototype */


void blending_wrapper(int, double *); /* wrapper for the algorithm */



/*ECG PROCESSING ROUTINES */

void read(int *,int *,int *, int *, int, int *);			/* to read the ecg data from file (coded as int between 0 and 1023) converted by the function in int between (-511 +511) so to make signed operations possible */

void difference_first(int *,int *);	/*first part of computation fo the difference signal (i.e. computation of X_d) */

void difference_second(int *);		/*second parto of the computation of the difference signal (i.e. computation of x_d_f) */

int Rpeackdetect(int * ,int, int, int *, int *); /*detect The R peaks */

int Q_point_detect(int *, int  , int );   /*detect the Q points */

int Q_deciding(int ,  int ,  int *);	/* given two candidate Q points, decide which one is the "best" */

int peakfinder( int *,  int );		/*just an algorithm finding the maximum of an array */

void filtering1984( int *);		/* smoothing filtering function */

void filter_base_line( int *);		/*base line drift filter */	

int cmpfunc (const void *, const void *);	/*just coompare two input numbers and return a positive value if the first is bigger, a negative values if thw second is bigger  */

int Twave_detect( int * , int, const int );		/* local search for the T peak detection and TRA method to find Tend */

double Detect( int *,  int *);			/*main function of the detect algorithm from which all the others function are called */

int find_kth( int *, int, int, int);  /*it finds the kth element of an array (used in the mdian computation), recursive algorithm*/

int partition( int *, int, int );       /*auxilary function for the above find_kth */

int mean_QT();	/* return the mean of the qt_intervals global array*/

int median_QT();	/* return the median of the qt_intervals global array*/

int QT_wrap();    /* wrap for mean and median computation of QT*/       

double pacing( int);	/* return the computed RR values in seconds*/

double linear_pacing(double); /*Linear pacing model*/

double bezzet_pacing(double); /*Bezzet pacing model*/

double hyperbolic_pacing(double qt); /*Hyperbolic pacing model*/

double fridericia_pacing(double); /*Fridericia pacing model*/

double sarma_pacing(double); /*Sarma pacing model*/
 
void inizialization_rr(void); /* used to inizialize the global vector R_points_loc_global at the begin of each iteration*/

void inizialization_qt(void); /* used to inizialize the global vector QT_intervals_global at the begin of the FIRST iteration*/

double RR_interval(); /*Computes the actual RR intervals mean/median detected */

int RR_interval_discrete();

void insertion_sort(int *, int );

/*ACCELEROMETER PROCESSING ROUTINES */

int mean_computation(int *, int );		/*compute the mean of an array */

void filtering_acc_singal(int *, int *); /*A simple normalising filter used in the accelerometer processing algorithm */

double accelerometer_processing(int *, int *, int *); /*Main function for the accelerometer processing routine */

double DAC_converter(int , int );		/*Convert a digital number to an analog one, use the macros of the Signal libraries*/

/*Blending routines */

double blending(double , double );		/*Routine used to blend the pacing from the two sensors */

double blend_aux(double , double );	/*Auxiliary routine for the blending one */

void initElistArrays(void);

double evalGauss(double x,double elist[ELIST_SAVED][ColumnElist],double HeightL[ELIST_SAVED][ColumnElist],double WidthL[ELIST_SAVED][ColumnElist],int mins,int maxs);

double *SynthECG(void);

void SynthEcgWrap(int *);

int minpeakfinder(int *X , int len);

#define BIN_MAX(a, b) ((a >= b) ? a : b) /* finding maximum between two numbers */

/* ERRORS USING POINTERS */
#define THROW_ERROR	{ fprintf(stderr,"An error occured in function %s, stopping simulation.... \n\n",__func__); fflush(stderr);exit(1);}

/*Random numbers generator   */
#define randomize srand((unsigned)time(NULL))

#define random(x) rand() % x

/*GLOBAL VARIABLES */
 					/*flag_for_over indicates if there is the need to reuse some points in the input array in the successive step. see SECURE_WINDOW */   
int R_points_loc_global[QT_SAVED];
int R_points_loc_global_old[QT_SAVED];
int QT_intervals_global[QT_SAVED];
int global_index_QT;
int global_index_R_loc;
int count_flag;

/*Global arrays used to keep track of the heart events needed for generation of synthethic signal */
double ElistColumn1[ELIST_SAVED];
double ElistColumn2[ELIST_SAVED];
double ElistColumn3[ELIST_SAVED];
double ElistColumn4[ELIST_SAVED];
double idxElist4=0;
/* */
#include "../blending_wrapper.c"
#include "../DAC_converter.c"
#include "../Detect.c"
#include "../Q_deciding.c"
#include "../Q_detect.c"
#include "../Rpeakdetect.c"
#include "../T_detect.c"
#include "../Accelerometer_processing.c"
#include "../mean_computation.c"
#include "../Filtering_accelerometer.c"
#include "../Blending.c"
#include "../difference.c"
#include "../BaseLineFilter.c"
#include "../filtering84.c"
#include "../Init_global.c"
#include "../Array_utils.c"
#include "../pacing.c"
#include "../read.c"
#include "../QT_wrap.c"
#include "../RR_wrap.c"
#include "../SynthECG.c"
